/*
 * Address Conversion Tool Tester
 *
 * Organization : OGCIO(F)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 0.2
*/

import java.io.*;
import java.util.*;
import java.text.NumberFormat;
import hksarg.ogcio.AddressConversion.*;
import hksarg.ogcio.AddressConversion.ConversionRule.ConvertAddressException;

public class TextFileAddressTester
{
	public TextFileAddressTester()
	{
	}
	
	public static void main(String[] argv)
	{
		if (argv.length != 2)
		{
			System.out.println("Usage: java TextFileAddressTester rvd_input output");
			System.exit(1);
		}
		
		TextFileAddressTester tester = new TextFileAddressTester();
		tester.convert(argv[0],argv[1]);
	}
	
	public void convert(String inputFile, String outputFile)
	{
		Vector addressLines = new Vector();
		int outputDimension[];
		
		BufferedReader input = null;
		PrintWriter out = null;
		int numAddressRead = 0;
		int numAddressConvertSuccess = 0;
		int numAddressConvertFail = 0;
		
		long startTime = System.currentTimeMillis();
		
		// read input file
		try
		{
			input = new BufferedReader(new InputStreamReader(new FileInputStream(inputFile),"Big5-HKSCS"));
			String lineBuffer = null;
			// read output dimension
			// read number of lines
			lineBuffer=input.readLine();
			int dimensionLine = Integer.parseInt(lineBuffer);
			outputDimension = new int[dimensionLine];
			for (int i=0;i<dimensionLine;i++)
			{
				lineBuffer=input.readLine();
				outputDimension[i] = Integer.parseInt(lineBuffer);
			}
			
			// open output file
			out = new PrintWriter(new BufferedWriter(new OutputStreamWriter(new FileOutputStream(outputFile),"Big5-HKSCS")));
			
			// read input addresses
			while ((lineBuffer = input.readLine()) != null)
			{
				String line = lineBuffer.trim();
				if (line.length()!=0)
					addressLines.add(line);
				else
				{
					if (addressLines.size() > 0)
					{
						String[] source = (String[])addressLines.toArray(new String[addressLines.size()]);
						numAddressRead++;
						String[] dest = null;
						String errMsg = null;
						try
						{
							dest = convertAddress(source,outputDimension);
							numAddressConvertSuccess++;
						}
						catch (ConvertAddressException e)
						{
							numAddressConvertFail++;
							errMsg = " --- Convert exception: " + e.getMessage() + " --- ";
							dest = source;
						}
						outputResult(out,dest,errMsg);
					}
					addressLines.removeAllElements();
				}
			}
			// boundary condition, EOF reached
			if (addressLines.size() > 0)
			{
				String[] source = (String[])addressLines.toArray(new String[addressLines.size()]);
				numAddressRead++;
				String[] dest = null;
				String errMsg = null;
				try
				{
					dest = convertAddress(source,outputDimension);
					numAddressConvertSuccess++;
				}
				catch (ConvertAddressException e)
				{
					numAddressConvertFail++;
					errMsg = " --- Convert exception: " + e.getMessage() + " --- ";
					dest = source;
				}
				outputResult(out,dest,errMsg);
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if (input!=null)
					input.close();
			}
			catch (IOException ex)
			{
				ex.printStackTrace();
			}
			if (out!=null)
			{
				out.flush();
				out.close();
			}
		}
		
		long endTime = System.currentTimeMillis();
		printOverallResult(endTime-startTime,numAddressRead,numAddressConvertSuccess,numAddressConvertFail);
	}
	
	protected String[] convertAddress(String[] source,int[] outputDimension)
	throws ConvertAddressException
	{
		// concat address into single line for testing
		//String[] source1Line = new String[1];
		//source1Line[0] = "";
		//for (int i=0;i<source.length;i++)
		//	source1Line[0] = source1Line[0] + " " + source[i];
		//FreeTextAddressConvertor convertor =
		//		new FreeTextAddressConvertor(source1Line,outputDimension);
						
		FreeTextAddressConvertor convertor =
				new FreeTextAddressConvertor(source,outputDimension);
		return convertor.convert();
	}
	
	protected void outputResult(PrintWriter out, String[] address, String errMsg)
	{
		if (errMsg != null)
			out.println(" --- " + errMsg + " --- ");
		if (address == null)
		{
			out.println(" --- Destination is null --- ");
			return;
		}
		for (int i=0;i<address.length;i++)
		{
			out.println(address[i]);
		}
		out.println();
	}
	
	protected void printOverallResult(
			long timeSpent,
			int addressRead,
			int convertSuccess,
			int convertFail)
	{
		System.out.print("Total execution time: ");
		if (timeSpent > 86400000)
		{
			System.out.print(timeSpent/86400000 + " Day ");
			timeSpent = timeSpent % 86400000;
		}
		if (timeSpent > 3600000)
		{
			System.out.print(timeSpent/3600000 + "h");
			timeSpent = timeSpent % 3600000;
		}
		if (timeSpent > 60000)
		{
			System.out.print(timeSpent/60000 + "m");
			timeSpent = timeSpent % 60000;
		}
		NumberFormat form = NumberFormat.getInstance();
		form.setMaximumFractionDigits(3);
		form.setMinimumFractionDigits(0);
		System.out.println(form.format(timeSpent/1000.0) + "s");
		
		System.out.println("Number of address read: " + addressRead);
		System.out.println("Number of address successfully converted: " + convertSuccess);
		if (convertFail > 0)
			System.out.println("Number of address failed: " + convertFail);
	}
}
