/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion;

import java.util.*;
import hksarg.ogcio.AddressConversion.ConversionRule.*;

public class FreeTextAddressConvertor
{
	public static final boolean DEBUG = false;
	
	protected FreeTextAddress source;
	protected int[] outputDimension;
	boolean isDimensionChanged;
	RuleEngine ruleEngine;
	
	public FreeTextAddressConvertor()
	{
		source = null;
		outputDimension = null;
		isDimensionChanged = true;
		ruleEngine = new RuleEngine();
	}
	
	public FreeTextAddressConvertor(String[] address)
	{
		source = new FreeTextAddress(address);
		defaultOutputDimension();
		ruleEngine = new RuleEngine();
	}
	
	public FreeTextAddressConvertor(String[] address, int[] outputDimension)
	{
		source = new FreeTextAddress(address);
		this.outputDimension = outputDimension;
		isDimensionChanged = true;
		ruleEngine = new RuleEngine();
	}
	
	public void defaultOutputDimension()
	{
		outputDimension = new int[5];
		for (int i=0;i<5;i++)
			outputDimension[i] = 60;
		isDimensionChanged = true;
	}
	
	public void setSourceAddress(String[] address)
	{
		source = new FreeTextAddress(address);
	}
	
	public void setOutputDimension(int[] outputDimension)
	{
		this.outputDimension = outputDimension;
		isDimensionChanged = true;
	}
	
	public String[] convert()
	throws ConvertAddressException, IllegalArgumentException
	{
		if (source==null)
			throw new IllegalArgumentException("null source address");
		if (outputDimension==null)
			defaultOutputDimension();
		if (isDimensionChanged)
		{
			ruleEngine.clearRule();
			RuleFactory ruleFactory = new RuleFactory();
			ruleEngine.addRule(ruleFactory.getRules(outputDimension));
		}
		
		FreeTextAddress output = ruleEngine.fire(source);
		return output.getAddress();
	}
}
