/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion;

import java.util.*;
import hksarg.ogcio.AddressConversion.ConversionRule.*;

public class RuleEngine
{
	protected static int MAX_LOOP_COUNT = 100;
	protected Vector rules;
	
	public RuleEngine()
	{
		rules = new Vector();
	}
	
	public void addRule(Rule rule)
	{
		rules.add(rule);
	}
	
	public void addRule(Rule[] rule)
	{
		int size = rule.length;
		for (int i=0;i<size;i++)
			rules.add(rule[i]);
	}
	
	public void clearRule()
	{
		rules.clear();
	}
	
	public FreeTextAddress fire(FreeTextAddress input)
	throws ConvertAddressException
	{
		boolean flag = true;
		int loopCount=0;
		int ruleSize = rules.size();
		FreeTextAddress addr = new FreeTextAddress(input);
		while (flag)
		{
			boolean ruleFired = false;
			if (((Rule)rules.get(0)).fire(addr))
			{
				break;
			}
			
			for (int i=1;i<ruleSize;i++)
			{
				if (((Rule)rules.get(i)).fire(addr))
				{
					ruleFired = true;
					if (FreeTextAddressConvertor.DEBUG)
					{
						System.out.println("Rule " + i + " fired");
					}
					break;
				}
			}
			if (ruleFired == false)
			{
				if (!addr.isForceBreakBetweenWord())
				{
					if (FreeTextAddressConvertor.DEBUG)
					{
						System.out.println("Force break between word");
					}
					addr.setForceBreakBetweenWord();
					continue;
				}
				else
				{
					throw new ConvertAddressException("No rule fired, the source does not fit into destination");
				}
			}
			if (++loopCount > MAX_LOOP_COUNT)
			{
				throw new ConvertAddressException("Maximum loop count encountered");
			}
		}
		return addr;
	}
}
