/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 03/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseBuildingFuzzyToken extends FuzzyToken
{
	private static final String DEFAULT_TOKEN = "[xHOaӐhbթ~Gxp].*?";
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"j\\s*[H].*?","J\\s*.*?","[sӰ]\\s*.*?","\\s*.*?","[]\\s*|.*?"
			,"O\\s*.*?","s\\s*.*?","\\s*.*?","X\\s*p.*?","s\\s*.*?"
			,"[jp]\\s*.*?","l\\s*"};
	
	private static final double LINGUISTIC_DECAY = 0.6;
	private static final double LINGUISTIC_DECAY_INCREMENT = 0.05;
	private static final double LINGUISTIC_DECAY_MAXIMUM = 0.9;
	private static final double SINGLE_TOKEN_LINGUISTIC_START = 0.9;
	private static final double DOUBLE_TOKEN_LINGUISTIC_START = 1.0;
	
	protected ChineseBuildingFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		try
		{
			Pattern p = Pattern.compile(DEFAULT_TOKEN);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				return new ChineseBuildingFuzzyToken(wordArray,wordPosition,0);
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		
		if (wordPosition > wordArray.length-2)
			return null;
		for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_2_WORD_TOKEN[i],Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
				if(m.matches())
				{
					return new ChineseBuildingFuzzyToken(wordArray,wordPosition,1);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int extendCount)
	{
		if (extendCount > 0)
			linguistic[wordPosition] = DOUBLE_TOKEN_LINGUISTIC_START;
		else
			linguistic[wordPosition] = SINGLE_TOKEN_LINGUISTIC_START;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		double linguisticDecay = LINGUISTIC_DECAY;
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= linguisticDecay;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
			if (linguisticDecay < LINGUISTIC_DECAY_MAXIMUM)
				linguisticDecay += LINGUISTIC_DECAY_INCREMENT;
		}
		
		for (int j=1;j<=extendCount;j++)
		{
			linguistic[wordPosition+j] = linguistic[wordPosition];
		}
	}
}
