/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 26/07/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseDdLotFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_1_WORD_TOKEN = 
			{""};
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"[al]\\s*q"};
	private static final String[] DEFAULT_3_WORD_TOKEN = 
			{"p\\s*\\s*q"};
	private static final String[] DEFAULT_4_WORD_TOKEN = 
			{"[V]\\s*q\\s*\\s*"};
	private static final double DEFAULT_AREA_LINGUISTIC = 1.0;
	private static final double LINGUISTIC_DECAY_FRONT = 0.47;
	private static final double LINGUISTIC_DECAY_END = 0.40;
		
	protected ChineseDdLotFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-4)
		{
			for (int i=0;i<DEFAULT_4_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(DEFAULT_4_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition]
							+ wordArray[wordPosition+1]
							+ wordArray[wordPosition+2]
							+ wordArray[wordPosition+3]);
					if(m.matches())
					{
						return new ChineseDdLotFuzzyToken(wordArray,wordPosition,3);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-3)
		{
			for (int i=0;i<DEFAULT_3_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(DEFAULT_3_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2]);
					if(m.matches())
					{
						return new ChineseDdLotFuzzyToken(wordArray,wordPosition,2);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseDdLotFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		for (int i=0;i<DEFAULT_1_WORD_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_1_WORD_TOKEN[i]+".*?");
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new ChineseDdLotFuzzyToken(wordArray,wordPosition,0);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int extendCount)
	{
		int i;
		for (i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
		
		double currentLinguistic = DEFAULT_AREA_LINGUISTIC * LINGUISTIC_DECAY_FRONT;
		i = wordPosition-1;
		if (i>0 && containNumber(wordArray[i]))
		{
			linguistic[i] = currentLinguistic;
		}
		
		// if the case of Vq q, no need to include the words follow
		if (extendCount != 3)
		{
			currentLinguistic = DEFAULT_AREA_LINGUISTIC * LINGUISTIC_DECAY_END;
			i=wordPosition+extendCount+1;
			if (i<wordArray.length)
			{
				if (ChineseNumberPrefixFuzzyToken.getInstance(wordArray,i)!=null)
				{
					linguistic[i] = currentLinguistic;
				}
			}
		}
	}
}
