/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 02/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseFloorFuzzyToken extends FuzzyToken
{
	private static final String DEFAULT_TOKEN = "h.*?";
	private static final String SPECIAL_TOKEN = ".*?";
	private static final String SPECIAL_TOKEN_PREFIX = "[r].*?";
	
	private static final double DEFAULT_LINGUISTIC = 1.0;
	private static final double LINGUISTIC_DECAY = 0.33;
	
/*	protected ChineseFloorFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
*/
	
	protected ChineseFloorFuzzyToken(String[] wordArray, int wordPosition, int skipWord, boolean digitsOnly)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,skipWord,digitsOnly);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		try
		{
			Pattern p = Pattern.compile(DEFAULT_TOKEN,Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				return new ChineseFloorFuzzyToken(wordArray,wordPosition,0,false);
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		try
		{
			Pattern p = Pattern.compile(SPECIAL_TOKEN,Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				// normally, "" is treated as ChineseBuildingFuzzyToken
				// exception when the word before contains number or xxr or ռ
				if (wordPosition - 1 >= 0)
				{
					Pattern specialPattern = Pattern.compile(SPECIAL_TOKEN_PREFIX,Pattern.CASE_INSENSITIVE);
					Matcher specialMatcher = specialPattern.matcher(wordArray[wordPosition-1]);
					if (specialMatcher.matches())
					{
						return new ChineseFloorFuzzyToken(wordArray,wordPosition,1,true);
					}
					if (FuzzyToken.containNumber(wordArray[wordPosition-1]))
					{
						return new ChineseFloorFuzzyToken(wordArray,wordPosition,0,true);
					}
				}
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int skipWord, boolean digitsOnly)
	{
		int i;
		double currentLinguistic = DEFAULT_LINGUISTIC;
		try
		{
			for (i=0;i<=skipWord;i++)
				linguistic[wordPosition-i] = currentLinguistic;
		}
		catch (ArrayIndexOutOfBoundsException ex)
		{
		}
		
		for (i=wordPosition-1-skipWord;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
			{
				if (digitsOnly)
				{
					if (containNumber(wordArray[i])
							|| (ChineseNumberFuzzyToken.getInstance(wordArray,i) != null))
						linguistic[i] = currentLinguistic;
					else
						break;
				}
				else
				{
					linguistic[i] = currentLinguistic;
				}
			}
			else
			{
				break;
			}
		}
	}
}
