/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 06/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseHKDistrictFuzzyToken extends GroupBasedFuzzyToken
{
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"\\s*","[W]\\s*","[n]\\s*W","[b]\\s*s","[_]\\s*"
			,"\\s*W","j\\s*[|HųDV]","s\\s*","W\\s*J"
			,"m\\s*i","\\s*","[[o]\\s*","[ŷsFA]\\s*","\\s*I"
			,"\\s*","\\s*","[ŭx]\\s*a","[|F]\\s*f","\\s*[F]"
			,"j\\s*}","[ѫnd]\\s*","K\\s*m","W\\s*|","[W]\\s*w"
			,"\\s*^","s\\s*V","`\\s*[}]"
			,"W\\s*","p\\s*V"
			,"\\s*[Mlj]","\\s*","\\s*[E]"
			,"C\\s*[w]","\\s*","\\s*"
			,"\\s*[wF]","\\s*","\\s*W"
			,"\\s*[^D]","\\s*","[FnĮ]\\s*F"
			,"[Fn_]\\s*","\\s*|","[N]\\s*D","[]\\s*","\\s*"
			,"q\\s*J","\\s*V","\\s*[]","\\s*d","ù\\s*","\\s*L"
			,"[LH]\\s*","\\s*","t\\s*s","[Ww]\\s*Y","\\s*H"
			,"\\s*","\\s*","\\s*","\\s*","n\\s*","\\s*B"
			,"_\\s*Y","\\s*Z","\\s*d","\\s*w"};
	private static final String[] DEFAULT_3_WORD_TOKEN = 
			{"\\s*\\s*J","n\\s*[cQ]\\s*w","\\s*r\\s*W","\\s*\\s*"
			,"[`LM]\\s*\\s*W","]\\s*\\s*a","\\s*\\s*L","o\\s*\\s*F"
			,"\\s*W\\s*e","\\s*\\s*L","K\\s*\\s*W"
			,"\\s*\\s*C","\\s*\\s*|"
			,"\\s*G\\s*","[]\\s*F\\s*W","p\\s*\\s*s","\\s*\\s*"
			,"E\\s*s\\s*[W]","\\s*K\\s*"
			,"\\s*Y\\s*","\\s*\\s*W","\\s*Y\\s*","s\\s*Z\\s*^"
			,"q\\s*Z\\s*W","`\\s*\\s*","\\s*\\s*","j\\s*\\s*[CL]"
			,"g\\s*\\s*W","y\\s*F\\s*C","O\\s*\\s*s","\\s*Y\\s*"
			,"\\s*j\\s*P","o\\s*[w]\\s*a","S\\s*@\\s*"
			,"x\\s*\\s*","E\\s*{\\s*s","y\\s*B\\s*s"
			,"\\s*\\s*","\\s*b\\s*s","F\\s*Y\\s*","p\\s*w\\s*"
			,"\\s*\\s*C","\\s*\\s*","\\s*\\s*","N\\s*x\\s*D"
			,"\\s*[Py]\\s*"
			,"\\s*\\s*W","r\\s*\\s*W","\\s*F\\s*w","n\\s*X\\s*q"
			,"j\\s*\\s*s","p\\s*\\s*W"
			,"[F]\\s*E\\s*s","j\\s*H\\s*[VY]]","\\s*x\\s*s"
			,"W\\s*\\s*F","P\\s*H\\s*","\\s*\\s*W","\\s*\\s*w"
			,"j\\s*Q\\s*W","C\\s*s\\s*Y","_\\s*\\s*W","\\s*l\\s*s"
			,"U\\s*\\s*","\\s*\\s*w","b\\s*s\\s*","\\s*\\s*W"
			,"_\\s*\\s*s","\\uDF40\\s*\\s*W","F\\s*\\s*W","`\\s*}\\s*W"
			,"\\s*\\s*w","s\\s*\\s*","q\\s*J\\s*","\\s*U\\s*D"
			,"y\\s*\\s*T","\\s*\\s*","s\\s*D\\s*Y","\\s*a\\s*s"
			,"M\\s*X\\s*","\\s*\\s*s","\\s*\\s*H","\\s*P\\s*"
			,"Q\\s*\\s*F","F\\s*Y\\s*W","]\\s*\\s*w","³\\s*\\s*W"
			,"p\\s*\\s*L","_\\s*\\s*F","Q\\s*|\\s*m"
			,"\\s*\\s*","j\\s*F\\s*[f]","j\\s*\\s*J","\\s*s\\s*F"
			,"U\\s*\\s*|","D\\s*\\s*s","\\s*\\s*b"
			,"\\s*A\\s*","\\s*\\s*V","W\\s*\\s*[m]"
			,"Q\\s*\\s*","Q\\s*\\s*J","\\s*\\s*","j\\s*\\s*"
			,"\\s*\\s*","W\\s*\\s*y","\\s*T\\s*s","T\\s*\\s*"
			,"o\\s*a\\s*Y","C\\s*s\\s*W","s\\s*\\s*y","\\s*o\\s*s"
			,"\\s*\\s*s","j\\s*V\\s*F","j\\s*p\\s*","\\s*\\s*"
			,"x\\s*\\s*","\\s*\\s*","\\s*\\s*","Q\\s*K\\s*m"
			,"\\s*x\\s*s","j\\s*\\s*f","\\s*\\s*","\\s*\\s*h"
			,"\\s*W\\s*","\\s*\\s*q","F\\s*\\s*","\\s*@\\s*"
			,"j\\s*\\s*|"};
	private static final String[] DEFAULT_4_WORD_TOKEN = 
			{"j\\s*\\s*\\s*","\\s*\\s*a\\s*","j\\s*}\\s*\\s*d"
			,"b\\s*C\\s*b\\s*q","E\\s*s\\s*|\\s*s","\\s*\\s*\\s*V"};
	
	private static final double DEFAULT_AREA_LINGUISTIC = 0.26;
	
	protected ChineseHKDistrictFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-4)
		{
			for (int i=0;i<DEFAULT_4_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_4_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3]);
					if(m.matches())
					{
						return new ChineseHKDistrictFuzzyToken(wordArray,wordPosition,3);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-3)
		{
			for (int i=0;i<DEFAULT_3_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_3_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2]);
					if(m.matches())
					{
						return new ChineseHKDistrictFuzzyToken(wordArray,wordPosition,2);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseHKDistrictFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		for (int i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
	}
}
