/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 03/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseLocationFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"W"};
	
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"\\s*"};
	
	private static final double LINGUISTIC_DEFAULT = 0.41;
	private static final double LINGUISTIC_DECAY = 0.63;
	
	protected ChineseLocationFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseLocationFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new ChineseLocationFuzzyToken(wordArray,wordPosition,0);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		linguistic[wordPosition] = LINGUISTIC_DEFAULT;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			
			if (currentLinguistic >= DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
		
		for (i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = LINGUISTIC_DEFAULT;
		}
	}
}
