/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 04/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChinesePOBoxFuzzyToken extends FuzzyToken
{
	private static final String DEFAULT_2_WORD_TOKEN = 
			"[Hl]\\s*c";
	private static final String DEFAULT_4_WORD_TOKEN = 
			"\\s*\\s*l\\s*c";
	private static final double DEFAULT_LINGUISTIC = 0.21;
	private static final double LINGUISTIC_DECAY = 0.505;
	
	protected ChinesePOBoxFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-4)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_4_WORD_TOKEN + ".*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3]);
				if(m.matches())
				{
					return new ChinesePOBoxFuzzyToken(wordArray,wordPosition,3);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_2_WORD_TOKEN + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
				if(m.matches())
				{
					return new ChinesePOBoxFuzzyToken(wordArray,wordPosition,1);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		int i;
		for (i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_LINGUISTIC;
		}
		
		double currentLinguistic = DEFAULT_LINGUISTIC;
		for (int j=wordPosition+i;j<linguistic.length;j++)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			
			if (currentLinguistic >= DEFAULT_WORD_LINGUISTIC)
				linguistic[j] = currentLinguistic;
			else
				break;
		}
	}
}
