/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseSectionFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"\\s*q"};
	private static final double DEFAULT_AREA_LINGUISTIC = 1.0;
	private static final double LINGUISTIC_DECAY = 0.47;
		
	protected ChineseSectionFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseSectionFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		int i;
		for (i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
		
		double currentLinguistic = DEFAULT_AREA_LINGUISTIC;
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
	}
}
