/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 26/07/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseSpecialGroupFuzzyToken extends GroupBasedFuzzyToken
{
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"a\\s*[Uw]","\\s*"};
	private static final String[] DEFAULT_5_WORD_TOKEN = 
			{"\\s*{\\s*[]\\s*Q\\s*|"};
	private static final double DEFAULT_AREA_LINGUISTIC = 0.2;
	
	protected ChineseSpecialGroupFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-5)
		{
			for (int i=0;i<DEFAULT_5_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_5_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3] + wordArray[wordPosition+4]);
					if(m.matches())
					{
						return new ChineseSpecialGroupFuzzyToken(wordArray,wordPosition,4);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseSpecialGroupFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		for (int i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
	}
}
