/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 03/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class ChineseStreetFuzzyToken extends FuzzyToken
{
	private static final String DEFAULT_PATTERN = "[D|{Ѿq].*?";
	private static final String DIRECTION_PATTERN = "[Fn_].*?";
	
	private static final String[] DEFAULT_2_WORD_TOKEN =
			{"F\\s*u","\\s*Y"};
	private static final String[] DEFAULT_3_WORD_TOKEN =
			{"\\s*\\s*B"};
	
	// should test ChineseStreetFuzzyToken before ChineseBuildingFuzzyToken in FuzzyTokenFactory
	// exception E  [ xM
	// aֹD aw^D Ө  bF JĤ@ܤ
	// HshF Hsh
	private static final String[] EXCEPTION_3_WORD_STREET = 
			{"\\s*[E[]\\s*[]","x\\s*M\\s*"};
	private static final String[] EXCEPTION_4_WORD_STREET = 
			{"a\\s*(\\s*|w\\s*^)\\s*D", "\\s*\\s*\\s*"
			, "[b]\\s*\\s*[F]\\s*"};
	private static final String[] EXCEPTION_5_WORD_STREET = 
			{"\\s*J\\s*\\s*[@GT|]\\s*", "H\\s*s\\s*h\\s*[F]\\s*"};
	
	private static final double LINGUISTIC_DECAY = 0.635;
	
	// tokenException = true, means the street is EXCEPTION_X_WORD_STREET
	protected ChineseStreetFuzzyToken(String[] wordArray, int wordPosition, int extendCount, boolean tokenException)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,extendCount,tokenException);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		try
		{
			Pattern p = Pattern.compile(DEFAULT_PATTERN);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				return new ChineseStreetFuzzyToken(wordArray,wordPosition,0,false);
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		
		if (wordPosition <= wordArray.length-3)
		{
			for (int i=0;i<DEFAULT_3_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_3_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2]);
					if(m.matches())
					{
						return new ChineseStreetFuzzyToken(wordArray,wordPosition,2,false);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+DEFAULT_2_WORD_TOKEN[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new ChineseStreetFuzzyToken(wordArray,wordPosition,1,false);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		// test for exception cases
		if (wordPosition <= wordArray.length-3)
		{
			for (int i=0;i<EXCEPTION_3_WORD_STREET.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+EXCEPTION_3_WORD_STREET[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2]);
					if(m.matches())
					{
						return new ChineseStreetFuzzyToken(wordArray,wordPosition,2,true);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		if (wordPosition <= wordArray.length-4)
		{
			for (int i=0;i<EXCEPTION_4_WORD_STREET.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+EXCEPTION_4_WORD_STREET[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3]);
					if(m.matches())
					{
						return new ChineseStreetFuzzyToken(wordArray,wordPosition,3,true);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		if (wordPosition <= wordArray.length-5)
		{
			for (int i=0;i<EXCEPTION_5_WORD_STREET.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile(".*?"+EXCEPTION_5_WORD_STREET[i]+".*?");
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3] + wordArray[wordPosition+4]);
					if(m.matches())
					{
						return new ChineseStreetFuzzyToken(wordArray,wordPosition,4,true);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int extendCount, boolean exact)
	{
		linguistic[wordPosition] = 1.0;
		
		int i, startPosition;
		double currentLinguistic = linguistic[wordPosition];
		if (exact)
		{
			for (i=1;i<=extendCount;i++)
				linguistic[wordPosition+i] = 1.0;
			startPosition = wordPosition+i-1;
		}
		else
		{
			for (i=wordPosition-1;i>=0;i--)
			{
				currentLinguistic *= LINGUISTIC_DECAY;
				
				if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
					linguistic[i] = currentLinguistic;
				else
					break;
			}
			startPosition = wordPosition;
		}
		currentLinguistic = linguistic[startPosition];
		
		/* can be xxn214 */
		boolean directionalToken = false;
		for (i=startPosition+1;i<wordArray.length;i++)
		{
			if (isDirectionToken(wordArray[i]))
			{
				/* handle the cases like ӦZjDjH */
				if (!directionalToken && ChineseLocationFuzzyToken.getInstance(wordArray,i)==null)
				{
					currentLinguistic *= LINGUISTIC_DECAY;
					linguistic[i] = currentLinguistic;
					directionalToken = true;
				}
				else
					break;
			}
			else
			{
				if (containNumber(wordArray[i])
						|| ChineseNumberFuzzyToken.getInstance(wordArray,i)!=null)
				{
					currentLinguistic *= LINGUISTIC_DECAY;
					linguistic[i] = currentLinguistic;
				}
				break;
			}
		}
	}
	
	private boolean isDirectionToken(String word)
	{
		try
		{
			Pattern p = Pattern.compile(DIRECTION_PATTERN);
			Matcher m = p.matcher(word);
			return m.matches();
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		return false;
	}
}
