/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 15/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishBlockFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"bl(oc)?ks?","ph(ase)?","stages?","su?ites?"};
	
	// special token, there should be no word in front of this token
	// otherwise it will be treated as EnglishBuildingFuzzyToken
	private static final String SPECIAL_HOUSE_TOKEN = "h(ou)?ses?";
	
	// TOWER is moved to EnglishKeywordTowerFuzzyToken
	
	private static final double FIRST_LINGUISTIC_VALUE = 0.5;
	private static final double LINGUISTIC_DECAY = 0.21;
	
	protected EnglishBlockFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishBlockFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		// test special token
		// normally HOUSE belongs to EnglishBuildingFuzzyToken
		// however, if it is the first word in an address line, it will become a EnglishBlockFuzzyToken
		// e.g.  House 1 Block E Flamingo Gardens Sai Kung
		if (wordPosition == 0)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + SPECIAL_HOUSE_TOKEN + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishBlockFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = 1.0;
		
		int i;
		double currentLinguistic = FIRST_LINGUISTIC_VALUE;
		for (i=wordPosition+1;i<wordArray.length;i++)
		{
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
			currentLinguistic *= LINGUISTIC_DECAY;
		}
	}
}
