/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 03/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishBuildingFuzzyToken extends FuzzyToken
{
	// keyword TOWER is moved to FuzzyKeywordTowerFuzzyToken
	private static final String[] DEFAULT_TOKEN = 
			{"ap(artmen)?ts?","arc(ade)?","bazaar","building","BL(VD|DG)"
			,"cent(re|er)","CTR","(ci|f)ty","CR?T","co(urt|ve|mplex)"
			,"est(ate)?","factory","gardens?","GDNs?","h(eigh)?ts?"
			,"hotel","h(ou)?ses?","(land)?mark(et)?","lau","lodge","mall","mans(ion)?s?","MKT"
			,"offices?","park","pla(ce|za)","quarters?","QTRs?","rise","reclamation","sq(uare)?"
			,"terrace","villa?s?","village","VLG","(ch|ts|y)uen","racecourse"};
	
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"race\\s+course"};
	
	private static final double DEFAULT_LINGUISTIC = 1.0;
	private static final double LINGUISTIC_DECAY = 0.57;
		
	protected EnglishBuildingFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishBuildingFuzzyToken(wordArray,wordPosition,0);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile("\\W*?" + DEFAULT_2_WORD_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new EnglishBuildingFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int extendCount)
	{
		double currentLinguistic = DEFAULT_LINGUISTIC;
		int i;
		
		for (i=0;i<=extendCount;i++)
			linguistic[wordPosition+i] = currentLinguistic;
		
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
	}
}
