/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishDdLotFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"(D|S|MWD)D","LOTs?","CLL","K?IL","(sub)?sections?"};
	
	private static final double LINGUISTIC_DECAY = 0.35;
	
	protected EnglishDdLotFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishDdLotFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = 1.0;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition+1;i<wordArray.length;i++)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
	}
}
