/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 26/07/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishFloorFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{".+/fl?","fl(oor)?s?","levels?"};
	
	private static final double LINGUISTIC_DECAY = 0.33;
	
	protected EnglishFloorFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishFloorFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = 1.0;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		
		// e.g. 34/F
		try
		{
			Pattern p = Pattern.compile(DEFAULT_TOKEN[0] + "\\W*?",Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				return;
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
		
		currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition+1;i<wordArray.length;i++)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			// the word after FLOOR must be a number
			// other it will not be treated as part of EnglishFloorFuzzyToken
			if (!containNumber(wordArray[i]))
				break;
			else if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
	}
}
