/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 06/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishHKDistrictFuzzyToken extends GroupBasedFuzzyToken
{
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"CAUSEWAY\\s+BAY","(CHAI|MA|(TS|SH)UEN)\\s+WAN","HAPPY\\s+VALLEY"
			,"KENNEDY\\s+TOWN"
			,"MID(-|\\s)+LEVELS?","NORTH\\s+POINT","(QUARRY|REPULSE)\\s+BAY","SHEK\\s+(O|KONG|PIK|MUN)"
			,"SHEUNG\\s+(WAN|SHUI)","THE\\s+PEAK","WAN\\s+CHAI","WESTERN\\s+DISTRICT"
			,"CHOI\\s+HUNG","(FORTRESS|DIAMOND)\\s+HILL","HUNG\\s+HOM","K(OWLOO|L)N\\s+(BAY|CITY|TONG)"
			,"(KWUN|YAU)\\s+TONG","LAM\\s+(TIN|TEI|TSUEN)","LOK\\s+FU","MONG\\s+KOK"
			,"FO\\s+TAN","(FU|KWAN)\\s+TEI","(CHU|HA)NG\\s+HAU","TUNG\\s+CHUNG","KWU\\s+TUNG"
			,"(LO|NAI)\\s+WAI","PAT\\s+HEUNG","PING\\s+(CHE|SHAN)","SAI\\s+(KUNG|WAN)"
			,"SAN\\s+(HUI|TIN)","([HK]AM|PAK|SHA)\\s+TIN","SHAM\\s+(WAT|TSENG)"
			,"SIU\\s+(LAM|HONG)"
			,"TAI\\s+(HANG|TAM|LAM|PO|TONG|WAI|WO|O|KOO)"
			,"TAM\\s+MEI","TING\\s+(KAU|KOK)","TSING\\s+YI"
			,"TUEN\\s+MUN","YUEN\\s+LONG"
			,"CHEUNG\\s+(CHAU|SHA)","(JUNK|DISCOVERY)\\s+BAY"
			,"(CROOKED|GRASS|GREEN|LAMMA|LANTAU|STONECUTTERS)\\s+ISLAND"
			,"MUI\\s+WO","(NG|L)ONG\\s+PING","(P[EI]|WA)NG\\s+CHAU","TONG\\s+FUK"
			,"FA\\s+PENG","JARDINE'?S?\\s+LOOKOUT","PUI\\s+O","SHUI\\s+HANG"
			,"MIU\\s+TSAI","LUK\\s+KENG","WANG\\s+(LE|TO)NG","MOUNT\\s+DAVIS","TIN\\s+(LI|HA)U"
			,"LO\\s+WU","YING\\s+PUN","CHUEN\\s+LUNG","(CH)?AU\\s+TAU","HA\\s+TSUEN"
			,"MAI\\s+PO","KWAI\\s+(CHUNG|HING|FONG)","LAI\\s+KING"
			,"MEI\\s+FOO","PRINCE\\s+EDWARD","NAM\\s+(WA[NI]|CH(U|EO)NG)"
			,"PO\\s+LAM","CITY\\s+ONE","HENG\\s+ON"};
	private static final String[] DEFAULT_3_WORD_TOKEN = 
			{"AP\\s+LEI\\s+CHAU","CHUNG\\s+HOM\\s+KOK","(CLEAR|DEEP)\\s+WATER\\s+BAY"
			,"POK\\s+FU\\s+LAM","SAI\\s+WAN\\s+HO","SAI\\s+YING\\s+PUN","SHAU\\s+KEI\\s+WAN"
			,"SHEK\\s+TONG\\s+TSUI","WONG\\s+CHUK\\s+HANG"
			,"CHA\\s+KWO\\s+LING","(CHEUNG|PAK)\\s+SHA\\s+WAN","HO\\s+MAN\\s+TIN","LAI\\s+CHI\\s+KOK"
			,"MA\\s+TAU\\s+WAI","NGAU\\s+CHI\\s+WAN","NGAU\\s+TAU\\s+KOK","SAN\\s+PO\\s+KONG"
			,"SAU\\s+MAU\\s+PING","SHAM\\s+SHUI\\s+PO","SHEK\\s+KIP\\s+MEI","TAI\\s+KOK\\s+TSUI"
			,"TO\\s+KWA\\s+WAN","TSIM\\s+SHA\\s+TSUI","TSZ\\s+WAN\\s+SHAN","WANG\\s+TAU\\s+HOM"
			,"WONG\\s+TAI\\s+SIN","YAU\\s+MA\\s+TEI","YAU\\s+YAT\\s+CHUEN"
			,"HUNG\\s+SHUI\\s+KIU","KAU\\s+TO\\s+SHAN"
			,"LAU\\s+FAU\\s+SHAN","MA\\s+LIU\\s+SHUI","MA\\s+ON\\s+SHAN","SHA\\s+TAU\\s+KOK"
			,"SIU\\s+LEK\\s+YUEN","SO\\s+KWUN\\s+WAT","TA\\s+KWU\\s+LING","TIN\\s+SHUI\\s+WAI"
			,"TSEUNG\\s+KWAN\\s+O"
			,"CHEK\\s+LAP\\s+KOK","CHI\\s+MA\\s+WAN","HEI\\s+LING\\s+CHAU"
			,"SIU\\s+SAI\\s+WAN","LEI\\s+YUE\\s+MUN","LOK\\s+MA\\s+CHAU","NGA\\s+KAU\\s+WAN"
			,"PAK\\s+KOK\\s+SHAN","SAN\\s+SHEK\\s+WAN","SHA\\s+LO\\s+WAN"
			,"SHAM\\s+WAT\\s+WAN","SHEK\\s+KWU\\s+CHAU","SHEUNG\\s+KWAI\\s+CHUNG"
			,"SIN\\s+YAN\\s+TSENG","SOK\\s+KWU\\s+WAN","TAI\\s+KWAI\\s+WAN"
			,"TSING\\s+LUNG\\s+TAU","YUNG\\s+SHUE\\s+WAN","MANG\\s+KUNG\\s+UK"
			,"MIU\\s+TSAI\\s+TUN","PO\\s+TOI\\s+O","LAU\\s+SHUI\\s+HEUNG"
			,"LO\\s+SHUE\\s+LING","LUNG\\s+YEUK\\s+TAU","SZE\\s+TEI\\s+SHAN"
			,"WO\\s+HOP\\s+SHEK","WU\\s+TIP\\s+SHAN","SO\\s+KON\\s+PO","WU\\s+KAI\\s+SHA"
			,"TUNG\\s+WAN\\s+TAU","KAU\\s+SAI\\s+SHAU","LEUNG\\s+SHUEN\\s+WAN"
			,"LITTLE\\s+PALM\\s+BEACH","PAK\\s+TAM\\s+CHUNG","SHAP\\s+(SZE|PAT)\\s+HEUNG"
			,"TA\\s+KU\\s+LING","TAI\\s+CHUNG\\s+(HAU|KIU)","TAI\\s+MONG\\s+TSAI"
			,"HA\\s+LUNG\\s+CHUNG","HA\\s+WO\\s+CHEE"
			,"TO\\s+FUNG\\s+SHAN","HOK\\s+TSUI\\s+PENINSULA","MA\\s+TSO\\s+LUNG"
			,"MAN\\s+KAM\\s+TO","SHEK\\s+WU\\s+HUI","SHEUNG\\s+SHUI\\s+(HEUNG|WAI)"
			,"TAI\\s+PO\\s+(KAU|MARKET)","WU\\s+KAU\\s+TANG","TIM\\s+TIN\\s+TSAI"
			,"HUNG\\s+MUI\\s+KUK","TAI\\s+NGAU\\s+WU","FUNG\\s+SHUE\\s+WO"
			,"SHEUNG\\s+KO\\s+TAN","FU\\s+YUNG\\s+SHAN","SAM\\s+DIP\\s+TAM"
			,"YAU\\s+KOM\\s+TAU","CASTLE\\s+PEAK\\s+BAY","HUNG\\s+SHUI\\s+KIU"
			,"LUNG\\s+KWU\\s+TAN","MOR\\s+LAW\\s+SHAN","POR\\s+LO\\s+SHAN"
			,"TAI\\s+LAM\\s+CHUNG","TAI\\s+SIU\\s+FAN","CHU\\s+WONG\\s+LING"
			,"NGAU\\s+TAM\\s+MEI","SAI\\s+PIN\\s+WAI","WANG\\s+TOI\\s+SHAN"
			,"TAI\\s+WO\\s+HAU","TIU\\s+KENG\\s+LENG","HENG\\s+FA\\s+CHUEN"
			,"TSUEN\\s+WAN\\s+WEST","CHE\\s+KUNG\\s+TEMPLE","SHA\\s+TIN\\s+WAI"
			,"TAI\\s+SHUI\\s+HANG"};
	private static final String[] DEFAULT_4_WORD_TOKEN = 
			{"TAI\\s+TSOI\\s+YUEN\\s+KUI","TAI\\s+TUNG\\s+WO\\s+LIU"
			,"HAU\\s+LUNG\\s+HANG\\s+SHAN","YUEN\\s+LONG\\s+KAU\\s+HUI"};
	
	private static final double DEFAULT_AREA_LINGUISTIC = 0.2;
	
	protected EnglishHKDistrictFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition <= wordArray.length-4)
		{
			for (int i=0;i<DEFAULT_4_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile("\\W*?" + DEFAULT_4_WORD_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2] + wordArray[wordPosition+3]);
					if(m.matches())
					{
						return new EnglishHKDistrictFuzzyToken(wordArray,wordPosition,3);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-3)
		{
			for (int i=0;i<DEFAULT_3_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile("\\W*?" + DEFAULT_3_WORD_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1] + wordArray[wordPosition+2]);
					if(m.matches())
					{
						return new EnglishHKDistrictFuzzyToken(wordArray,wordPosition,2);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		if (wordPosition <= wordArray.length-2)
		{
			for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
			{
				try
				{
					Pattern p = Pattern.compile("\\W*?" + DEFAULT_2_WORD_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
					Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
					if(m.matches())
					{
						return new EnglishHKDistrictFuzzyToken(wordArray,wordPosition,1);
					}
				}
				catch (PatternSyntaxException e)
				{
					e.printStackTrace();
				}
			}
		}
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		for (int i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
	}
}
