/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishHKRegionFuzzyToken extends GroupBasedFuzzyToken
{
	private static final String[] DEFAULT_2_WORD_TOKEN = 
			{"HONG\\s+KONG","H\\.?\\s*K","NEW\\s+TERRITORIES","N\\.?\\s*T"};
	private static final double DEFAULT_AREA_LINGUISTIC = 0.25;
	
	protected EnglishHKRegionFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordPosition,extendCount);
		skipWordNumber = extendCount;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition > wordArray.length-2)
			return null;
		for (int i=0;i<DEFAULT_2_WORD_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_2_WORD_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition] + wordArray[wordPosition+1]);
				if(m.matches())
				{
					return new EnglishHKRegionFuzzyToken(wordArray,wordPosition,1);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(int wordPosition, int extendCount)
	{
		for (int i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_AREA_LINGUISTIC;
		}
	}
}
