/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 15/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

// Handle cases like WEST GATE TOWER or TOWER 1 THE GATEWAY
public class EnglishKeywordTowerFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"t(owe|w)rs?"};
	
	// Patterns for number (e.g. 1, 2, I, II, VI)
	// or a single character, or english number (e.g. one, two)
	private static final String[] NUMBER_PATTERN = 
			{"([IVX]+|[a-zA-Z])","(one|two|three|four|five|six|seven|eight|nine|ten|eleven|twelve)"};
	
	private static final double DEFAULT_LINGUISTIC = 1.0;
	private static final double LINGUISTIC_DECAY = 0.33;
	private static final double LINGUISTIC_VALUE_WORD_AFTER = 0.33;
	
	// true if the word after is digits
	// false otherwise
	// the variable is initialized in isNumber()
	protected boolean isWordAfterDigits;
		
	protected EnglishKeywordTowerFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishKeywordTowerFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = DEFAULT_LINGUISTIC;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
		
		i = wordPosition+1;
		if (i<wordArray.length)
		{
			if (isNumber(wordArray[i]))
				linguistic[i] = LINGUISTIC_VALUE_WORD_AFTER;
		}
	}
	
	// for the case of TOWER A or TOWER 10
	// a word followed by TOWER can be a number (e.g. 1, 2, I, II, VI)
	// or a single character, or english number (e.g. one, two)
	private boolean isNumber(String word)
	{
		if (containNumber(word))
		{
			isWordAfterDigits = true;
			return true;
		}
		for (int i=0;i<NUMBER_PATTERN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(NUMBER_PATTERN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(word);
				if (m.matches())
				{
					isWordAfterDigits = false;
					return true;
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return false;
	}
	
	// precondition: the word after TOWER may belongs to street
	// if more than 1 number following TOWER, the number immediately after TOWER may belongs to TOWER
	// so the linguistic value of the word after TOWER remains the same
	// if only 1 number is following TOWER, it treat as house number of street
	// so reset the linguistic value of the word after TOWER to 0.0
	// it is only true if the word after is numeric digits
	public void resetLinguisticValueAfter(int numDigitFollow)
	{
		if (!isWordAfterDigits || numDigitFollow > 1)
			return;
		for (int i=linguistic.length-1;i>=0;i--)
		{
			if (linguistic[i] == DEFAULT_LINGUISTIC)
			{
				if (i+1<linguistic.length)
					linguistic[i+1] = 0.0;
				break;
			}
		}
	}
}
