/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 16/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishLocationFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"bay","beach","praya","wan"};
	
	private static final double LINGUISTIC_DEFAULT = 0.41;
	private static final double LINGUISTIC_DECAY = 0.5;
	
	protected EnglishLocationFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishLocationFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = LINGUISTIC_DEFAULT;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			
			if (currentLinguistic >= DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
	}
}
