/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 15/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishRoomFlatFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"fla?ts?","r(oo)?ms?","s(hop|tall|uite)s?","units?"};
	
	private static final double LINGUISTIC_DECAY = 0.33;
	
	protected EnglishRoomFlatFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile("\\W*?" + DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishRoomFlatFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = 1.0;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition+1;i<wordArray.length;i++)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				linguistic[i] = currentLinguistic;
			else
				break;
		}
		
		//currentLinguistic = linguistic[wordPosition];
		//for (i=wordPosition-1;i>=0;i--)
		//{
		//	currentLinguistic *= LINGUISTIC_DECAY;
		//	if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
		//		linguistic[i] = currentLinguistic;
		//	else
		//		break;
		//}
	}
}
