/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 23/07/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class EnglishStreetFuzzyToken extends FuzzyToken
{
	private static final String[] DEFAULT_TOKEN = 
			{"alley","ave(nue)?","b(oulevar|lv)d","bridge","circuit","close","corridor","cres(cent)?"
			,"dr(ive)?","embankment","highway","lane","interchange","parade","path","promenade"
			,"r(oa)?d","row","st(reet|rand)?","tunnel","walk"};
	private static final String[] DIRECTION_TOKEN =
			{"(nor|sou)th","(ea|we)st","central","[neswc]"};
			//{"north","n","east","e","south","s","west","w","central","c"};
	private static final String SPECIAL_STREET_TOKEN = "queensway";
	
	private static final double LINGUISTIC_DECAY = 0.635;
	private static final double LINGUISTIC_DIRECTION_DECAY = 0.19;
	
	protected EnglishStreetFuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition);
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		for (int i=0;i<DEFAULT_TOKEN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(DEFAULT_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(wordArray[wordPosition]);
				if(m.matches())
				{
					return new EnglishStreetFuzzyToken(wordArray,wordPosition);
				}
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		
		try
		{
			Pattern p = Pattern.compile(SPECIAL_STREET_TOKEN + "\\W*?",Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(wordArray[wordPosition]);
			if(m.matches())
			{
				return new EnglishStreetFuzzyToken(wordArray,wordPosition);
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition)
	{
		linguistic[wordPosition] = 1.0;
		
		int i;
		double currentLinguistic = linguistic[wordPosition];
		for (i=wordPosition-1;i>=0;i--)
		{
			currentLinguistic *= LINGUISTIC_DECAY;
			linguistic[i] = currentLinguistic;
			
			// street token will end when reaching house number of the street
			// i.e. 30 ABC ROAD
			// the case 28-30 ABC ROAD is handled by RangeFuzzyToken
			if (containNumber(wordArray[i]))
				break;
			else if (currentLinguistic > DEFAULT_WORD_LINGUISTIC)
				continue;
			else
				break;
		}
		currentLinguistic = linguistic[wordPosition] * LINGUISTIC_DIRECTION_DECAY;
		
		if (wordPosition+1 < wordArray.length)
		{
			if (isDirectionToken(wordArray[wordPosition+1]))
				linguistic[wordPosition+1] = currentLinguistic;
		}
	}
	
	private boolean isDirectionToken(String word)
	{
		boolean flag = false;
		try
		{
			for (int i=0;i<DIRECTION_TOKEN.length;i++)
			{
				Pattern p = Pattern.compile("\\W*?" + DIRECTION_TOKEN[i] + "\\W*?",Pattern.CASE_INSENSITIVE);
				Matcher m = p.matcher(word);
				flag = m.matches();
				if (flag)
					break;
			}
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		return flag;
	}
}
