/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 15/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public class RangeFuzzyToken extends GroupBasedFuzzyToken
{
	private static final String DEFAULT_TOKEN = "(\\W*NOs?\\W*?|.*-|.*&|.*w|.*)\\s?";
	
	private static final double DEFAULT_LINGUISTIC = GroupBasedFuzzyToken.DEFAULT_LINGUISTIC_MINIMUM;
	
	protected RangeFuzzyToken(String[] wordArray, int wordPosition, int extendCount)
	throws ArrayIndexOutOfBoundsException
	{
		super(wordArray,wordPosition);
		init(wordArray,wordPosition,extendCount);
		skipWordNumber = extendCount-1;
	}
	
	public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	{
		if (wordPosition > wordArray.length-2)
			return null;
		try
		{
			Pattern p = Pattern.compile(DEFAULT_TOKEN,Pattern.CASE_INSENSITIVE);
			int i=0;
			Matcher m = p.matcher(wordArray[wordPosition+i]);
			while (m.matches() && wordPosition+i < wordArray.length)
			{
				i++;
				m = p.matcher(wordArray[wordPosition+i]);
			}
			if (i>0)
				return new RangeFuzzyToken(wordArray,wordPosition,i);
		}
		catch (PatternSyntaxException e)
		{
			e.printStackTrace();
		}
		return null;
	}
	
	private void init(String[] wordArray, int wordPosition, int extendCount)
	{
		for (int i=0;i<=extendCount;i++)
		{
			linguistic[wordPosition+i] = DEFAULT_LINGUISTIC;
		}
		if (wordPosition > 0)
		{
			// for the cases of 28 - 30 xxx street or Rms 4 & 5
			if (wordArray[wordPosition].charAt(0) == '-'
					|| wordArray[wordPosition].charAt(0) == '&')
			{
				linguistic[wordPosition-1] = DEFAULT_LINGUISTIC;
			}
		}
	}
}
