/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 02/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.regex.*;

public abstract class FuzzyToken
{
	public static final double DEFAULT_WORD_LINGUISTIC = 0.1;
	protected double[] linguistic;
	protected int skipWordNumber;
	protected boolean _isNumber;
	
	// check for 0-9 and Т
	private static String[] _CONTAIN_NUMBER_PATTERN = 
			{".*?\\d+.*?",".*?[\\uFF10\\uFF11\\uFF12\\uFF13\\uFF14\\uFF15\\uFF16\\uFF17\\uFF18\\uFF19]+.*?"};
	
	protected FuzzyToken(String[] wordArray, int wordPosition)
	throws ArrayIndexOutOfBoundsException
	{
		int arraySize = wordArray.length;
		if (wordPosition >= arraySize)
			throw new ArrayIndexOutOfBoundsException();
		linguistic = new double[arraySize];
		for (int i=0;i<arraySize;i++)
			linguistic[i] = 0.0;
		linguistic[wordPosition] = DEFAULT_WORD_LINGUISTIC;
		skipWordNumber = 0;
		_isNumber = containNumber(wordArray[wordPosition]);
	}
	
	//public static FuzzyToken getInstance(String[] wordArray, int wordPosition)
	//{
	//	return new FuzzyToken(wordArray,wordPosition);
	//}
	
	public double getLinguisticAt(int position)
	{
		return linguistic[position];
	}
	
	public int size()
	{
		return linguistic.length;
	}
	
	public int getSkipWordNumber()
	{
		return skipWordNumber;
	}
	
	public void setLinguisticAt(double value, int pos)
	{
		if (linguistic.length > pos)
		{
			linguistic[pos] = value;
		}
	}
	
	public boolean isNumber()
	{
		return _isNumber;
	}
	
	public static boolean containNumber(String word)
	{
		for (int i=0;i<_CONTAIN_NUMBER_PATTERN.length;i++)
		{
			try
			{
				Pattern p = Pattern.compile(_CONTAIN_NUMBER_PATTERN[i]);
				Matcher m = p.matcher(word);
				if(m.matches())
					return true;
			}
			catch (PatternSyntaxException e)
			{
				e.printStackTrace();
			}
		}
		return false;
	}
}
