/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 03/08/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import hksarg.ogcio.AddressConversion.FreeTextAddress;
import java.util.*;

public class FuzzyTokenFactory
{
	public FuzzyTokenFactory()
	{
	}
	
	public FuzzyToken[] getFuzzyToken(String[] wordArray)
	{
		int i;
		FuzzyToken[] token = new FuzzyToken[wordArray.length];
		for (i=0;i<wordArray.length;i++)
		{
			if (FreeTextAddress.isChineseUnicodeBlock(wordArray[i].charAt(0)))
			{
				token[i] = RangeFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseSpecialGroupFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseRoomFlatFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseFloorFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseBlockFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseStreetFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseDdLotFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseSectionFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseHKDistrictFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseHKRegionFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseBuildingFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseLocationFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseRangeFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseNumberFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseNumberPrefixFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChineseNumberUnitFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = ChinesePOBoxFuzzyToken.getInstance(wordArray,i);
				
				if (token[i]==null)
					token[i] = new DefaultFuzzyToken(wordArray,i);
			}
			else
			{
				token[i] = RangeFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishRoomFlatFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishFloorFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishBlockFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishStreetFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishDdLotFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishHKDistrictFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishHKRegionFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishBuildingFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishLocationFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishPOBoxFuzzyToken.getInstance(wordArray,i);
				if (token[i]==null)
					token[i] = EnglishKeywordTowerFuzzyToken.getInstance(wordArray,i);
				
				if (token[i]==null)
					token[i] = new DefaultFuzzyToken(wordArray,i);
			}
			
			if (token[i].getSkipWordNumber() > 0)
			{
				for (int j=1;j<=token[i].getSkipWordNumber();j++)
				{
					token[i+j] = new DefaultFuzzyToken(wordArray,i+j);
				}
				i+=token[i].getSkipWordNumber();
			}
		}
		
		// handle special cases
		processTokenDependency(token);
		// handle GroupBasedFuzzyToken, i.e. District, Region, etc
		processGroupBasedFuzzyToken(token);
		
		return token;
	}
	
	// handle special cases
	private void processTokenDependency(FuzzyToken[] token)
	{
		for (int i=0;i<token.length-1;i++)
		{
			if ((token[i] instanceof EnglishBuildingFuzzyToken)
					|| (token[i] instanceof ChineseBuildingFuzzyToken))
			{
				// BuildingFuzzyToken (both Chinese and English) cannot immediately followed by 
				// StreetFuzzyToken or another BuildingFuzzyToken
				if ((token[i+1] instanceof EnglishStreetFuzzyToken)
						|| (token[i+1] instanceof EnglishBuildingFuzzyToken)
						|| (token[i+1] instanceof ChineseStreetFuzzyToken)
						|| (token[i+1] instanceof ChineseBuildingFuzzyToken))
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
			}
			if ((token[i] instanceof EnglishStreetFuzzyToken)
					|| (token[i] instanceof ChineseStreetFuzzyToken))
			{
				// StreetFuzzyToken (both Chinese and English) cannot immediately followed by
				// BuildingFuzzyToken
				if ((token[i+1] instanceof EnglishBuildingFuzzyToken)
						|| (token[i+1] instanceof ChineseBuildingFuzzyToken))
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
			}
			if (token[i] instanceof EnglishKeywordTowerFuzzyToken)
			{
				// EnglishKeywordTowerFuzzyToken cannot immediately followed by EnglishBuildingFuzzyToken
				if (token[i+1] instanceof EnglishBuildingFuzzyToken)
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
				int numDigitCount = 0;
				// if followed by EnglishStreetFuzzyToken, the word after TOWER may belongs to EnglishStreetFuzzyToken
				for (int j=i+1;j<token.length;j++)
				{
					if (token[j] instanceof DefaultFuzzyToken)
					{
						if (token[j].isNumber())
							numDigitCount++;
						continue;
					}
					else if (token[j] instanceof EnglishStreetFuzzyToken)
					{
						EnglishKeywordTowerFuzzyToken tower = (EnglishKeywordTowerFuzzyToken)(token[i]);
						tower.resetLinguisticValueAfter(numDigitCount);
						break;
					}
					else
						break;
				}
			}
			if (token[i] instanceof EnglishLocationFuzzyToken)
			{
				// EnglishLocationFuzzyToken cannot immediately followed by EnglishKeywordTowerFuzzyToken,
				// EnglishBuildingFuzzyToken, or EnglishStreetFuzzyToken
				if ((token[i+1] instanceof EnglishKeywordTowerFuzzyToken)
						|| (token[i+1] instanceof EnglishBuildingFuzzyToken)
						|| (token[i+1] instanceof EnglishStreetFuzzyToken))
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
			}
			if (token[i] instanceof ChineseLocationFuzzyToken)
			{
				// ChineseLocationFuzzyToken cannot immediately followed by
				// ChineseBuildingFuzzyToken, or ChineseStreetFuzzyToken
				if ((token[i+1] instanceof ChineseBuildingFuzzyToken)
						|| (token[i+1] instanceof ChineseStreetFuzzyToken))
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
			}
			if (token[i] instanceof ChineseStreetFuzzyToken)
			{
				// ChineseStreetFuzzyToken cannot followed by another ChineseStreetFuzzyToken
				for (int j=i+1;j<token.length;j++)
				{
					if (token[j] instanceof ChineseStreetFuzzyToken)
					{
						resetTokenToDefault(token[i], i);
						break;
					}
					else
					{
						if (token[j].getLinguisticAt(j) != FuzzyToken.DEFAULT_WORD_LINGUISTIC)
							break;
					}
				}
			}
			if (token[i] instanceof ChineseRangeFuzzyToken)
			{
				// FuzzyToken before the ChineseRangeFuzzyToken cannot be a special token
				for (int j=i-1;j>=0;j--)
				{
					if (token[j] instanceof GroupBasedFuzzyToken)
					{
						if (token[j].getLinguisticAt(j) != FuzzyToken.DEFAULT_WORD_LINGUISTIC)
							break;
					}
					else
					{
						if (token[j].getLinguisticAt(j) == FuzzyToken.DEFAULT_WORD_LINGUISTIC)
							continue;
						else
						{
							resetTokenToDefault(token[i], i);
							break;
						}
					}
				}
			}
			if (token[i] instanceof ChineseBlockFuzzyToken)
			{
				// ChineseBlockFuzzyToken cannot immediately followed by ChineseBuildingFuzzyToken
				if (token[i+1] instanceof ChineseBuildingFuzzyToken)
				{
					resetTokenToDefault(token[i], i);
					continue;
				}
			}
		}
	}
	
	// reset all linguistic value of the token to 0
	// the linguistic value of position pos set to DEFAULT_WORD_LINGUISTIC
	private void resetTokenToDefault(FuzzyToken fuzzyToken, int pos)
	{
		for (int i=0;i<fuzzyToken.size();i++)
		{
			if (i==pos)
				fuzzyToken.setLinguisticAt(FuzzyToken.DEFAULT_WORD_LINGUISTIC,i);
			else
				fuzzyToken.setLinguisticAt(0.0,i);
		}
	}
	
	// reset the linguistic value so that the tokenizer will not break
	// inside a GroupBasedFuzzyToken class
	private void processGroupBasedFuzzyToken(FuzzyToken[] token)
	{
		Vector groups = mergeGroupBasedToken(token);
		int i,j;
		
		for (i=0;i<groups.size();i++)
		{
			GroupBasedFuzzyToken gbToken = (GroupBasedFuzzyToken)groups.elementAt(i);
			int start = gbToken.getLinguisticStartPosition();
			int end = gbToken.getLinguisticEndPosition();
			double maxLinguistic = token[0].getLinguisticAt(start);
			int maxTokenPosition = 0;
			
			// Find the token having the largest linguistic value
			for (j=0;j<token.length;j++)
			{
				for (int k=start;k<=end;k++)
				{
					if (token[j].getLinguisticAt(k) > maxLinguistic)
					{
						maxLinguistic = token[j].getLinguisticAt(k);
						maxTokenPosition = j;
					}
				}
			}
			
			// Set the linguistic value of the token
			for (j=start;j<=end;j++)
				token[maxTokenPosition].setLinguisticAt(maxLinguistic,j);
		}
	}
	
	private Vector mergeGroupBasedToken(FuzzyToken[] token)
	{
		Vector groups = new Vector(token.length);
		GroupBasedFuzzyToken currentToken = null;
		
		for (int i=0;i<token.length;i++)
		{
			if (token[i] instanceof GroupBasedFuzzyToken)
			{
				if (currentToken == null)
				{
					currentToken = (GroupBasedFuzzyToken)token[i];
					continue;
				}
				else
				{
					GroupBasedFuzzyToken gbToken = (GroupBasedFuzzyToken)token[i];
					int gbStart = gbToken.getLinguisticStartPosition();
					int gbEnd = gbToken.getLinguisticEndPosition();
					int currentStart = currentToken.getLinguisticStartPosition();
					int currentEnd = currentToken.getLinguisticEndPosition();
					
					// if currentToken and gbToken overlaps
					if (gbStart <= currentEnd && gbEnd >= currentStart)
					{
						for (int j=0;j<currentToken.size();j++)
						{
							if (currentToken.getLinguisticAt(j) < gbToken.getLinguisticAt(j))
								currentToken.setLinguisticAt(gbToken.getLinguisticAt(j),j);
						}
					}
					else
					{
						groups.add(currentToken);
						currentToken = gbToken;
					}
				}
			}
		}
		if (currentToken != null)
			groups.add(currentToken);
		
		return groups;
	}
}
