/*
 * Address Conversion Tool
 *
 * Organization : OGCIO(E)11A
 * Last modified: 10/06/2004
 *
 * @author TH HEUNG
 * @version 1.0
*/

package hksarg.ogcio.AddressConversion.Tokenizer;

import java.util.*;
import java.text.BreakIterator;
import hksarg.ogcio.AddressConversion.FreeTextAddressConvertor;

public class FuzzyTokenizer
{
	public FuzzyTokenizer()
	{
	}
	
	public String[] tokenizer(String line)
	{
		BreakIterator boundary = BreakIterator.getLineInstance();
		Vector wordArray = new Vector();
		boundary.setText(line);
		int start = boundary.first();
		for (int end=boundary.next();
				end!=BreakIterator.DONE;
				start=end, end=boundary.next())
		{
			wordArray.add(line.substring(start,end));
		}
		
		String[] stringArr = (String[])wordArray.toArray(new String[wordArray.size()]);
		FuzzyTokenFactory fuzzyTokenFactory = new FuzzyTokenFactory();
		FuzzyToken[] fuzzyToken = fuzzyTokenFactory.getFuzzyToken(stringArr);
		
		return clusterWord(fuzzyToken,stringArr,line);
	}
	
	protected String[] clusterWord(FuzzyToken[] fuzzyToken, String[] stringArr, String line)
	{
		Vector tokenArray = new Vector();
		int[] maxLinguisticPosition = new int[fuzzyToken[0].size()];
		int i;
		
		if (FreeTextAddressConvertor.DEBUG)
		{
			printFuzzyToken(fuzzyToken);
		}
		
		for (i=0;i<maxLinguisticPosition.length;i++)
		{
			//double maxLinguisticValue = fuzzyToken[0].getLinguisticAt(i);
			//maxLinguisticPosition[i] = 0;
			double maxLinguisticValue = FuzzyToken.DEFAULT_WORD_LINGUISTIC;
			maxLinguisticPosition[i] = -1;
			for (int j=0;j<maxLinguisticPosition.length;j++)
			{
				if (fuzzyToken[j].getLinguisticAt(i) > maxLinguisticValue)
				{
					maxLinguisticPosition[i] = j;
					maxLinguisticValue = fuzzyToken[j].getLinguisticAt(i);
				}
			}
		}
		
		int currentValue = maxLinguisticPosition[0];
		int start = 0;
		int end = stringArr[0].length();
		for (i=1;i<maxLinguisticPosition.length;i++)
		{
			if (maxLinguisticPosition[i] != currentValue)
			{
				tokenArray.add(line.substring(start,end).trim());
				start = end;
				currentValue = maxLinguisticPosition[i];
			}
			end += stringArr[i].length();
		}
		tokenArray.add(line.substring(start).trim());
		return (String[])tokenArray.toArray(new String[tokenArray.size()]);
	}
	
	protected void printFuzzyToken(FuzzyToken[] fuzzyToken)
	{
		for (int i=0;i<fuzzyToken.length;i++)
		{
			for (int j=0;j<fuzzyToken[i].size();j++)
			{
				System.out.print(fuzzyToken[i].getLinguisticAt(j) + ", ");
			}
			System.out.print("\n");
		}
	}
}
